/*
 * This file is part of libFirm.
 * Copyright (C) 2012 University of Karlsruhe.
 */

/**
 * @file
 * @brief       Should be included in all public firm headers at the end
 * @author      Matthias Braun
 */

/* close extern "C" block from begin.h,
 * we need this without include guard (see begin.h) */
#ifdef __cplusplus
}
#endif
